@@ -70,7 +70,7 @@ struct _SrnChat {
 };
 
 struct _SrnChatConfig {
-    bool log; // TODO
+    bool log;
     bool render_mirc_color;
     char *password;
     GList *auto_run_cmd_list;
